import IKGlobals from './../IKGlobals';
//
export default class IKResponse {
  constructor(statusCode, body) {
    this._statusCode = statusCode;
    this._body = body;
    //
    this._streamingOut = false; //internal -- flag to indicate streaming out has started and avoid double stream call
    //indeally it should be moved to initialization method as we dont' want them to be changed on the go and also give the false impression of getters with the current naming pattern
    this._isStream = false; //
    this._contextRawBody = false;
    this._throwOnErrors = false;
    //
    this._headers = {
      "Access-Control-Allow-Origin": "*",
      "Access-Control-Allow-Credentials": true,
      "Content-Type": "application/json"
    };
  }
  getCode() { return this._statusCode; }
  getBody() { return this._body; }
  appendIntoBody(key, value) { this._body[key] = value; }
  appendHeader(key, value) { this._headers[key] = value; }
  build(context, requestID, isBatch, transaction) {
    //Stream support
    if (this._streamingOut) return;
    if (this._isStream) return this._pipe(context, requestID);
    //Raw response support
    if (this._contextRawBody) return this.rawContext(context, requestID, transaction);
    
    //append default fields
    if (requestID && this._body) this.appendIntoBody('transactionID', requestID); //append transaction ID
    //build response
    let b = {
      statusCode: this._statusCode,
      ...(this._body ? { body: JSON.stringify(this._body) } : {}),
      headers: this._headers
    };
    //log response and respond to context
    transaction.logger.debug(b)
    if (!isBatch) context.succeed(b);
  }
  _pipe(context, requestID) {
    //Check if not streaming
    if (this._streamingOut) return;
    this._streamingOut = true;
    //build response
    let b = {
      statusCode: this._statusCode,
      body: this._body, headers: this._headers
    };
    //log response and respond to context
    context.succeed(b);
  }
  rawContext(context, requestID, transaction) {
    //append default fields
    if(requestID && this._body) this.appendIntoBody('transactionID', requestID); //append transaction ID
    //log response and respond to context
    transaction.logger.debug(this._body)
    if (this.getCode() <= 200 && this.getCode() <= 299) context.succeed(this._body);
    else {
      //
      function CustomError(body) {
        this.name = body && body.error ? body.error : 'UnknownError';
        this.message = JSON.stringify(body);
      }
      CustomError.prototype = new Error();
      //
      if (!this._throwOnErrors) context.fail(new CustomError(this._body));
      else throw (new CustomError(this._body));
    }
  }
}


/* Shortcuts */
export function IKMissingParamResponse(paramName) {
  console.warn(`Invalid request - Path parameter ${paramName} is missing.`);
  return (new IKResponse(400, { err: `Invalid request. Path parameter ${paramName} is missing.`, errCode: IKGlobals.ErrorCode_MissingParam }));
}
export function IKMissingQueryResponse(paramName) {
  console.warn(`Invalid request - Query parameter ${paramName} is missing.`);
  return (new IKResponse(400, { err: `Invalid request. Query parameter ${paramName} is missing.`, errCode: IKGlobals.ErrorCode_MissingParam }));
}
export function IKBadRequestResponse(msg, errCode, optBody) {
  console.warn(`Bad request - ${msg}`);
  return (new IKResponse(400, { err: msg, ...(errCode ? { errCode: errCode } : {}), ...(optBody || {}) }));
}
export function IKBadRequestResponseWithRollback(msg, errCode, optBody) {
  console.warn(`Bad request - ${msg}`);
  return (new IKResponse(400, { err: msg, rollback: true, ...(errCode ? { errCode: errCode } : {}), ...(optBody || {}) }));
}
export function IKUnauthorizedResponse(msg, errCode) {
  console.warn(`Denying request - ${msg}`);
  return (new IKResponse(401, { err: msg, ...(errCode ? { errCode: errCode } : {}) }));
}
export function IKSuccessResponse(msg) { return (new IKResponse(200, (msg ? msg : {}))); }
export function IKRedirectResponse(url) { 
  const resp = new IKResponse(302);
  resp.appendHeader('Location', url);
  return resp;
}
export function IKSuccessNoContentResponse() { return (new IKResponse(204, null)); }
export function IKSuccessStreamResponse(stream, contentType) {
  const resp = new IKResponse(200, (stream ? stream : {}));
  resp._isStream = true;
  resp.appendHeader('Connection', 'keep-alive');
  if (contentType) resp.appendHeader('Content-type', contentType);
  return resp;
}
export function IKStepFunctionResponse(body, optionalCode) {
  const resp = new IKResponse(optionalCode || 200, body);
  resp._contextRawBody = true;
  resp._throwOnErrors = true;
  return resp;
}
